#include "internal.mh"

/* some C source code used in this example */

extern int f(int);
int b;

int main(int argc, char **argv)
{
	int x = argc;
	return 0;
}



/* $main is the main equivalent of MetaC */
meta void $main(strg arg[])
{	
	/* The following sequence of declaration defines several metavariables
	 * and initializes them with code-structure-patterns. A 
	 * code-structure-pattern (or simply pattern) is a piece of syntactical
	 * correct code without bound variables. The binding is done at
	 * instantiation time. In consequence there is no type verification for
	 * code-structure-patterns at parse-time. 
	 */
	
	/* metavariable e of type expr is assigned an expression pattern */
	expr e = (expr)(a + b + c + x);

	/* metavariable c of type expr is assigned an expression pattern */
	expr c = (expr)f(a);

	/* metavariable i of type ident is assigned identifier named "x"
	 * (cast is necessary to change lvalue from strg to ident and
	 *  verify that the given string is lexical correct)
	 */
	ident i = (ident)"x";

	/* define a metavariable bt of metatype type and initialize its value
	 * with "int *".
	 */

	type bt = typeof(int *);
	/* define metavariable T with type type and derive the C type "int"
	 * from the metavariable bt. The typedef keyword is necessary to
	 * make clear we don't want to know the type of bt, but want to
	 * use the contents of bt.
	 */
	type T = typeof(typedef bt &);

	/* define a code-structure-pattern for a statement */
	/* note the special := syntax that similar to a label */
	to_add := {
		T a = f(i);
		x = e;
	}

	$add_epilogue(main,codeof to_add);
	/* Phase 1: replace metavariables of type type with matching type-specifier/declarator pairs
	 * replaces T with int
	 * Phase 2: rename metavariables of type ident
	 * renames i to x
	 * Phase 3: bind to local variables
	 * a is local; e->a will be bound local
	 * Phase 4: bind to metavariables of caller
	 * c is global and metavariable, will be resolve via metavariable
	 * e is resolve via metavariable; its pattern is instantiate recursivly
	 * Phase 5: bind to variables in extended scope
	 * b will be bound global
	 * x will be bound in extended scope of instantiation location
	 * Phase 6: check newly created code for type-errors
	 */
}

/* vim:tw=0
 * vim:ft=c
 */
