#include "vc_hacks.h"
/* This file demonstrates an example, how one can verify and fix an
 * often occuring problem.
 *
 * The return code of close is often ignored. The problem with this is
 * that the file could reside on an NFS share and therefore run out of
 * storage during close. In this case close returns -1 and sets errno to
 * ENOSPC. There are even more situations in which close might fail. In
 * consequence the returnvalue of close must not be ignored.
 * 
 * This metaprogram searches for situations were the return value of
 * close is ignored and replaces their occurances with a standard
 * resolution to this problem.
 */


#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

void valid_use_of_close(int fd)
{
	if (-1 == close(fd))
		perror("valid_use_of_close: close failed");
}

int main()
{
	int input;
	input = open("myfile",O_RDONLY);
	/* ... */
	close(input);
	return 0;
}


#include "internal.mh"

meta void $main(strg arg[])
{
	stmt m[], *s = m;
	expr e;
	pattern := close(e);
	s = $find_stmt(codeof pattern,(stmt)main,m);
	if (s == m) {
		$print("error no matches found");
		return;
	}
	s = m;
	while (*s) {
		substitute := 
			if (close(e) == -1)
				fprintf(stderr,"error closing file: %s\n",strerror(errno));
		$match_stmt(codeof pattern, *s);
		$warn("enhancing close in line " + $line_of(*s));
		$insert_after(codeof substitute,*s);
		$remove(*s);
		++s;
	}
}
